import pandas as pd
import numpy as np
import pypyodbc
import scipy.stats as ss

from sklearn.ensemble import RandomForestClassifier

def get_equal_groups(data_df,group_count,variable_name):

    groups_dict = {}
    total_size = 0
    group_count_now = 1

    data_small_df = data_df[[variable_name]]
    groups_df = data_small_df.groupby([variable_name]).size()
    groups_df = groups_df.reset_index()
    avg_group_size_fl= (groups_df.sum()/group_count)[0]

    for i in range(0,len(groups_df.index)):
        group_int = (((groups_df.iloc[[i],[0]]).values)[0])[0] 
        size_int  = (((groups_df.iloc[[i],[1]]).values)[0])[0] 
        total_size = total_size + size_int

        groups_dict.update({group_int:group_count_now})

        if total_size > avg_group_size_fl:
            total_size = 0
            group_count_now = group_count_now + 1 
    col_name = variable_name+'_gp'
    dict_ps = pd.Series(groups_dict)
    dict_df = dict_ps.to_frame()
    dict_df.columns = [col_name]
    
    temp_df = dict_df.reset_index(inplace=False)
    
    min_df = temp_df.groupby([col_name]).min().astype(str)
    max_df = temp_df.groupby([col_name]).max().astype(str)
    
    full_gp_name_df = pd.merge(min_df, max_df, left_index=True, right_index=True)
    full_gp_name_df[(col_name+'_rg')] = (full_gp_name_df['index_x']) + "-To-" + (full_gp_name_df['index_y'])
    return_df = pd.merge(full_gp_name_df, temp_df, left_on=col_name, right_on=col_name)
    return_df = return_df[[col_name,(col_name+'_rg'),'index']]
    return_df = return_df.set_index('index')
    
    return return_df


# In[5]:


def get_col_summary(data_df,variable,variable_name,label_variable_name):
    
    new1_df = data_df.groupby([variable_name]).count()
    new2_df = data_df.groupby([variable_name,label_variable_name]).count()
    
    t_df = (new2_df[variable] / new1_df[variable])
    t_df = t_df.reset_index()
    pivot_df = t_df.pivot(index=variable_name, columns=label_variable_name, values=variable)
    
    t1 = new1_df[label_variable_name].reset_index()
    t1['TotalSize'] = t1[label_variable_name] / t1[label_variable_name].sum()
    return_df = pd.merge(t1, pivot_df, left_on=variable_name, right_on=variable_name)
    
    return return_df


# In[6]:


def analyze_df(data_df, label_name,group_cnt_int):
    columns_list = data_df.columns.tolist()
    columns_list.remove(label_name)
    for entry in columns_list: 
        columnd_dtype_st = str(data_df[entry].dtypes)
        if columnd_dtype_st == 'int64':
            groups_df = get_equal_groups(data_df,group_cnt_int,entry)
            data_df = pd.merge(data_df, groups_df, left_on=entry, right_index=True)
            col_pivot = get_col_summary(data_df,entry,entry+'_gp_rg',label_name)
            print("===========================================================")
            print(col_pivot)
    #elif columnd_dtype_st == 'object':
    #    print('2')




def cramers_v(x, y):

    confusion_matrix = pd.crosstab(x,y)
    chi2 = ss.chi2_contingency(confusion_matrix)[0]
    n = confusion_matrix.sum().sum()
    phi2 = chi2/n
    r,k = confusion_matrix.shape
    phi2corr = max(0, phi2-((k-1)*(r-1))/(n-1))
    rcorr = r-((r-1)**2)/(n-1)
    kcorr = k-((k-1)**2)/(n-1)
    return np.sqrt(phi2corr/min((kcorr-1),(rcorr-1)))



def analyze_association(df,Label_column):
    final = []

    col_num_list = list(df.select_dtypes('number').columns)
    col_obj_list = list(df.select_dtypes('object').columns)
    col_cat_list = list(df.select_dtypes('category').columns)
    col_cat_list = col_cat_list + col_obj_list
    
    print('======================NumToNum======================')
    for col1 in col_num_list:
        for col2 in col_num_list:
            print(col2+"|"+col1)
            x = df[col1]
            y = df[col2]
            final.append(['cramers_v',col1,col2,cramers_v(x, y)])
            final.append(['cramers_v',col2,col1,cramers_v(y, x)])

    print('======================CatToNum======================')
    for col1 in col_cat_list:
        for col2 in col_num_list:
            print(col2+"|"+col1)
            x = df[col1]
            y = df[col2]
            final.append(['cramers_v',col1,col2,cramers_v(x, y)])
            final.append(['cramers_v',col2,col1,cramers_v(y, x)])
    
    print('======================CatToCat======================')
    for col1 in col_cat_list:
        for col2 in col_cat_list:
            print(col2+"|"+col1)
            x = df[col1]
            y = df[col2]
            final.append(['cramers_v',col1,col2,cramers_v(x, y)])
            final.append(['cramers_v',col2,col1,cramers_v(y, x)])
                        
    df = pd.DataFrame.from_records(final,columns=['algo','col1','col2','val'])
    
    df = df[df['algo'] == 'cramers_v']
    df = df.drop(['algo'], axis=1)
    df = df.drop_duplicates()
    
    DATA_RECORDS = df.pivot(index='col2', columns='col1', values='val')
    
    print (DATA_RECORDS[Label_column].sort_values(ascending=False)[:30], '\n')
    return DATA_RECORDS


def get_important_features(df,Label_Column): 
    
    # ================================================================================================
    # ================================================================================================	
    df.fillna(0, inplace=True)    
    features_df = df.drop(Label_Column,axis=1,inplace=False)
    labels_df = df[Label_Column]    
    onehot_df = pd.get_dummies(features_df)
    features_np = onehot_df.values.tolist()
    labels_np = labels_df.values.tolist()    
    # ================================================================================================
    # ================================================================================================	
    
    clf = RandomForestClassifier(n_estimators=100,n_jobs=-1,verbose=1)
    RandomForestClassifier

    clf = clf.fit(features_np,labels_np)
    
    importances = clf.feature_importances_   
    importances_df = pd.DataFrame(importances)
    
    column_names_df  = pd.DataFrame(features_df.columns.values.tolist())

    importances_df_final = pd.concat([column_names_df,importances_df], axis=1, join_axes=[column_names_df.index])
    importances_df_final.columns = ["Feature_Name", "Importance"]
    importances_df_final = importances_df_final.sort_values(by=['Importance'], ascending=False)
    print(importances_df_final.head(100))

    return importances_df_final


def analyze_correlation(df,Label_column):
    
    df_local = df
    
    label_type = str(df_local['Label'].dtypes)
    if (label_type != 'int64') and (label_type != 'float64'):  
        labels = df_local[Label_column].unique().tolist()
        mapping = dict( zip(labels,range(len(labels))) )
        df_local = df_local.replace({Label_column: mapping},inplace=False)
    
    numeric_features = df_local.select_dtypes(include=[np.number])
    corr = numeric_features.corr()
            
    return corr